package puntti;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.PrintStream;
import java.util.Arrays;
import java.util.Scanner;

/**
 * 
 * |------------------------------------------------------------------------|
 * | Luokan nimi:  Liikkeet                             | Avustajat:        |
 * |-------------------------------------------------------------------------
 * | Vastuualueet:                                      |                   |
 * |                                                    | - Liike           |
 * | - Pitää yllä liikkeitä. Osaa poistaa ja lisätä     |                   |
 * | liikkeen                                           |                   |
 * | - lukee ja kirjoittaa liikkeet tiedostoon          |                   |
 * | - osaa etsiä ja lajitella                          |                   |
 * |-------------------------------------------------------------------------
 * @author laspa
 * @version 5.3.2025
 *
 */
public class Liikkeet {
    
    private static final int MAX_LIIKKEITA = 5;
    
    private int lkm = 0;
    private Liike[] alkiot;
    
    
    /**
     * Alustava taulukko liikkeistä
     */
    public Liikkeet() {
        this.alkiot = new Liike[MAX_LIIKKEITA];
    }
    
    

    /**
     * Lisää uuden liikkeen tietorakenteeseen. Ottaa liikkeen omistukseensa.
     * @param liike Lisättävän liikkeen viite. Tietorakenne muuttuu omistajaksi.
     * <pre name="test">
     * #THROWS SailoException
     * Liikkeet liikkeet = new Liikkeet();
     * Liike kyykky1 = new Liike(), kyykky2 = new Liike(), kyykky3 = new Liike(), kyykky4 = new Liike(), kyykky5 = new Liike(), kyykky6 = new Liike();
     * liikkeet.getLkm() === 0;
     * liikkeet.lisaa(kyykky1); liikkeet.getLkm() === 1;
     * liikkeet.lisaa(kyykky2); liikkeet.getLkm() === 2;
     * liikkeet.lisaa(kyykky1); liikkeet.getLkm() === 3;
     * liikkeet.lisaa(kyykky3); liikkeet.getLkm() === 4;
     * liikkeet.lisaa(kyykky4); liikkeet.getLkm() === 5;
     * liikkeet.lisaa(kyykky5); liikkeet.getLkm() === 6;
     * liikkeet.lisaa(kyykky6); liikkeet.getLkm() === 7;
     * liikkeet.anna(0) === kyykky1;
     * liikkeet.anna(1) === kyykky2;
     * liikkeet.anna(2) === kyykky1; 
     * liikkeet.anna(1) == kyykky1 === false;
     * liikkeet.anna(1) == kyykky2 === true;
     * liikkeet.lisaa(kyykky1); liikkeet.getLkm() === 8;
     * liikkeet.lisaa(kyykky1); liikkeet.getLkm() === 9;  
     * </pre>
     */
    public void lisaa(Liike liike){

        if ( lkm >= alkiot.length ) {
            alkiot = Arrays.copyOf(alkiot, alkiot.length+10);
        }
        alkiot[lkm++] = liike;
    
    }
    
    
    /**
     * Palauttaa viitteen i:teen liikkeeseen
     * @param i monennen liikken viite halutaan
     * @return viite liikkeeseen jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella.
     */
    public Liike anna(int i) throws IndexOutOfBoundsException{
        if (i < 0 || lkm <= i)
                throw new IndexOutOfBoundsException("Ei sallittu indeksi: " + i);
        return alkiot[i];
        
    }
    
    
    
    /**
     * Palauttaa liikkeiden määrän
     * @return liikkeiden määrän
     */
    public int getLkm() {
        return lkm;
    }
    
  
    /**
     * Lukee käyttäjän tiedostosta
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen ei onnistu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException{
      String tiedNimi = hakemisto +"/liikkeet.dat";
      File tiedosto = new File(tiedNimi);
      
      try (Scanner fi = new Scanner(new FileInputStream(tiedosto))){
          while(fi.hasNext()) {
              String s = fi.nextLine();
              if (s == null || "".equals(s) || s.charAt(0) == ';') continue;
              Liike liike = new Liike();
              liike.parse(s);
              lisaa(liike);
          }
      }catch (FileNotFoundException e ) {
          throw new SailoException("Ei saa luettua tiedostoa " + tiedNimi);
      }
      
    }
    
    
    /**
     * @param hakemisto mihin hakemistoon tehdään uusi tiedosto
     * @throws SailoException jos tallentaminen ei onnistu
     */
    public void tallenna(String hakemisto) throws SailoException {
        File tiedosto = new File(hakemisto + "/liikkeet.dat");
        try (PrintStream fo = new PrintStream(new FileOutputStream(tiedosto, false))){
                for (int i=0; i<this.getLkm();i++) {
                    Liike liike = this.anna(i);
                    fo.println(liike.toString());
                }
                
        }catch (FileNotFoundException e) {
        
            throw new SailoException("Tiedosto "+ tiedosto.getAbsolutePath() + " ei löydy");
        }
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Liikkeet liikkeet = new Liikkeet();
        
        
        try {
            liikkeet.lueTiedostosta("batman");
        } catch (SailoException ex) {
            
            System.out.println(ex.getMessage());
        }
        
        
        Liike kyykky = new Liike();
        Liike kyykky2 = new Liike();
        
        kyykky.lisaaLiike();
        kyykky.taytaKyykky();
        kyykky2.lisaaLiike();
        kyykky2.taytaKyykky();
        
        try {
            
            liikkeet.lisaa(kyykky);
            liikkeet.lisaa(kyykky2);
            
            System.out.println("__________Liikkeet testi__________");
            
            for (int i = 0; i < liikkeet.getLkm(); i++) {
                Liike liike = liikkeet.anna(i);
                System.out.println("Liike indeksi " + i);
                liike.tulosta(System.out);
            }
            
        } catch (SailoException ex) {
            
            System.out.println(ex.getMessage());
        }
        
        try {
            liikkeet.tallenna("batman");
        } catch (SailoException e) {
            System.err.println(e.getMessage());
        }
       
        
        
    
    } 

}
